<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       smartdatasoft.com
 * @since      1.0.0
 *
 * @package    Edd_Discount_For_Envato_Customers
 * @subpackage Edd_Discount_For_Envato_Customers/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Edd_Discount_For_Envato_Customers
 * @subpackage Edd_Discount_For_Envato_Customers/admin
 * @author     SmartDataSoft <support@smartdatasoft.com>
 */
class Edd_Discount_For_Envato_Customers_Admin extends Edd_Discount_For_Envato_Customers_Provider{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	public $markups;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

		$this->markups = new Edd_Discount_For_Envato_Customers_Markup();

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Edd_Discount_For_Envato_Customers_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Edd_Discount_For_Envato_Customers_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/edd-discount-for-envato-customers-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Edd_Discount_For_Envato_Customers_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Edd_Discount_For_Envato_Customers_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/edd-discount-for-envato-customers-admin.js', array( 'jquery' ), $this->version, false );

	}

	function edddfe_registered_settings_tabs( $sections ) {
		$sections['edddfe-settings-section'] = __( 'EDD Discount for Envato Customers', 'edd-discount-for-enavto-customers' );
	
		return $sections;
	}
	public function edddfe_settings( $settings ) {
		$main_settings = array(
			array(
				'id'   => 'edddfe_header',
				'name' => '<strong>' . __( 'Discount for Enavto Customers with EDD Products', 'edd-discount-for-enavto-customers' ) . '</strong>',
				'desc' => '',
				'type' => 'header',
				'size' => 'medium'
			),
			array(
				'id' => 'edddfe_personal_token',
				'name' => __( 'Envato Personal Token', 'edd-discount-for-enavto-customers' ),
				'desc' => 'Your Envato personal Key. <a href="https://smartdatasoft.com/docs/discount-for-enavto-customers-with-edd-products/getting-started/installing/#envato_personal_token" target="_blank">See the doc </a> to know where to find your key.',
				'type' => 'text',
				'size' => 'regular',
			),
			array(
				'id' => 'edddfe_discount_percentage',
				'name' => __( 'Discount Percentage', 'edd-discount-for-enavto-customers' ),
				'type' => 'number',
				'desc' => __( 'Enter the discount percentage. 10 = 10%', 'edd-discount-for-enavto-customers' ),
				'size' => 'regular',
				'std'  => '50',
			),
			array(
				'id'   => 'edddfe_enable_mail',
				'name' => 'Emailing Settings',
 				'desc' => wp_kses_post('<a href="https://codecanyon.net/item/discount-for-enavto-customers-with-edd-products/38005681" target="_blank">Get Pro</a> to get notified everytime an envato customer generates a coupon code via email.'),
				'type' => 'descriptive_text',
			),
		);
		return array_merge( $settings, array('edddfe-settings-section' => $main_settings) );
	}

	public function edddfe_add_meta_boxes(){
		add_meta_box( 'edddfe_envato_product_ids', 'Plan for Envato Customers', [$this, 'edddfe_envato_product_ids_func'], 'download' );
	}

	public function edddfe_envato_product_ids_func(){
		global $post;
		$existing = $this->has_discount_on_product($post->ID, 'edd');
		$name = '';
		$ids = '';
			
		if(isset($existing['plan_names'])){
			$name = array_pop($existing['plan_names']);
		}
		if(isset($existing['envato_product_ids'])){
			$ids = implode(',',$existing['envato_product_ids']);
		}
		$fields = array(
			array(
				'name' => __( 'Plan Name', 'edd-discount-for-envato-customers' ),
				'id' => 'plan',
				'type' => 'text',
				'value' => $name,
				'placeholder' => __( 'Special Discount', 'edd-discount-for-envato-customers' ),
			),
			array(
				'name' => __( 'Envato Product Ids', 'edd-discount-for-envato-customers' ),
				'id' => 'envato_product_ids',
				'type' => 'text',
				'value' => $ids
			)
		);
		$args = array(
			'fields' => $fields,
			'vals' => $existing
		);

		$fields = apply_filters('edddfe_plan_pro_fields',$args);

		$this->markups->edddfe_plans_html($fields);	
	}

	public function edddfe_plan_pro_fields_callback($args){
		$args['fields'][] = array(
			'name' => __( 'Discount percentage', 'edd-discount-for-envato-customers' ),
			'id' => 'edddfe_pro_alert',
			'type' => 'html_content',
			'value' => wp_kses_post('<a href="https://codecanyon.net/item/discount-for-enavto-customers-with-edd-products/38005681" target="_blank">Get Pro and add specific discount percentage for plans</a>')
		);
		return $args;
	}

	public function edddfe_save_metas(){
		if(isset($_POST['edddfe_plan']) && isset($_POST['edddfe_envato_product_ids'])){
			global $post;
			$plan_name = $this->edddfe_sanitization($_POST['edddfe_plan']);
			$envato_ids = $this->edddfe_sanitization($_POST['edddfe_envato_product_ids']);
			$check_existence = $this->has_discount_on_product($post->ID, 'edd');
				
			if(!$check_existence){
				$plans = $this->edddfe_add_plan($post->ID, $plan_name, $envato_ids);
			}else{
				if(isset($check_existence['envato_product_ids'])){
					$envato_ids = explode(',',$envato_ids);
					$diff_ids = array();
					if(count($envato_ids) > count($check_existence['envato_product_ids'])){
						$diff_ids = array_diff($envato_ids,$check_existence['envato_product_ids']);
					}elseif(count($envato_ids) < count($check_existence['envato_product_ids'])){
						$diff_ids = array_diff($check_existence['envato_product_ids'],$envato_ids);
					}else{
						$this->edddfe_update_plan($plan_name, $envato_ids, $check_existence, $post->ID);
						return;
					}
					$add = !empty(array_intersect($diff_ids, $envato_ids));
					if($add){
						$plans = $this->edddfe_add_plan($post->ID, $plan_name, $diff_ids );
					}else{
						$plans = $this->edddfe_delete_plan($post->ID, $diff_ids);
					}
				}
			}
		}
	}
	private function edddfe_update_plan($plan_name, $envato_ids, $check_existence, $id){
		global $wpdb;
		$table_name = $wpdb->prefix . 'edddfe_plans';
		$exist_checking = array_pop($check_existence['plan_names']);
		if($plan_name != $exist_checking){
			$wpdb->update( 
				$table_name, 
				array( 
					'name' => $plan_name
				), 
				array( 
					'name' => $exist_checking
				) 
			);
		}
			
		if($envato_ids !=  $check_existence['envato_product_ids']){
			foreach ($envato_ids as $key => $value) {
				if($value != $check_existence['envato_product_ids'][$key] && !in_array($value, $check_existence['envato_product_ids'])){
					$wpdb->update( 
						$table_name, 
						array( 
							'envato_product_id' => $value
						), 
						array( 
							'envato_product_id' => $check_existence['envato_product_ids'][$key],
							'edd_product_id' => $id
						) 
					);	
				}
			}
		}
		do_action('edddfe_plans_update', $id);
	}
	private function edddfe_add_plan($edd_id, $plan_name, $envato_ids = array()){
		global $wpdb;
		if(!is_array($envato_ids)){
			$envato_ids = explode(',', $envato_ids);
		}
		$envato_ids = array_unique($envato_ids);
		$plans = array();
		foreach($envato_ids as $envato_id){
			do_action('edddfe_plans_before_add', $wpdb->insert_id);
			$wpdb->insert( 
				$wpdb->prefix . 'edddfe_plans', 
				array( 
					'name' => $plan_name, 
					'edd_product_id' => (int) $edd_id,
					'envato_product_id' => (int) $envato_id
				), 
				array( 
					'%s', 
					'%d',
					'%d'
				) 
			);
			$plans[] = $wpdb->insert_id;
			do_action('edddfe_plans_after_add', $wpdb->insert_id);
		}
		return $plans;
	}
	private function edddfe_delete_plan($edd_id, $env_ids){
		global $wpdb;
		if(!is_array($env_ids)){
			$env_ids = explode(',', $env_ids);
		}
		$env_ids = array_unique($env_ids);
		foreach($env_ids as $env_id){
			$wpdb->delete( 
				$wpdb->prefix . 'edddfe_plans', 
				array( 
					'edd_product_id' => (int) $edd_id,
					'envato_product_id' => (int) $env_id
				), 
				array( 
					'%d',
					'%d'
				) 
			);
		}
	}
}
